# InstantSearch Insights for iOS

**InstantSearch Insights iOS** library allows developers to capture search-related events. The events maybe related to search queries (such as click an conversion events used for Click Analytics or A/B testing). It does so by correlating events with queryIDs generated by the search API when a query parameter `clickAnalytics=true` is set. As well library allows to capture search-independent events which can be used for the purpose of search experience personalization. There are three types of these events which are currently supported: click, conversion and view.

## Quick Start

### Initialize the Insights client

You first need to initialize the Insights client. For that you need your **Application ID** and **API Key**.
You can find them on [your Algolia account](https://www.algolia.com/api-keys).
Also, for the purpose of personalization an application **User Token** can be specified via the corresponding optional parameter. In case of non-specified user token an automatically-generated application-wide user token will be used.

```swift
Insights.register(appId: "testApp", 
                  apiKey: "testKey", 
                  userToken: "testToken")
```

### Sending metrics

Once that you registered your **Application ID** and the **API Key** you can easily start sending metrics. 

```swift
Insights.shared?.clickedAfterSearch(eventName: "click event",
                                    indexName: "index name",
                                    objectID: "object id",
                                    position: 1,
                                    queryID: "query id")

Insights.shared?.convertedAfterSearch(eventName: "conversion event",
                                      indexName: "index name",
                                      objectIDs: ["obj1", "obj2"],
                                      queryID: "query id")

Insights.shared?.viewed(eventName: "view event",
                        indexName: "index name",
                        filters: ["brand:apple"])

```

### Event Batching

By default, events are only sent by batches of 10. You can customize this setting with `minBatchSize`:

```swift
Insights.minBatchSize = 1 // Sends each event as soon as it is tracked
```

### User tracking

Any event should have an userToken field to specify the user it relates to. You can set it in three ways:

- Globally for all events
- Per application, for every event tracked by this app
- Individually on an event

```swift
// Global userToken default value
Insights.shared?.userToken = "userToken"

// Application userToken, overrides global default
Insights.register(appId: "yourApplicationID",
                  apiKey: "yourAPIKey",
                  userToken: "userToken")

// Event userToken, overrides previous defaults
Insights.shared?.clicked(eventName: "eventName",
                         indexName: "indexName",
                         objectID: "objectID1",
                         userToken: "userToken")
```

### User opt-out

You should allow users to opt-out of tracking anytime they want to. When they request opt-out, you can honor it using:

```swift
Insights.shared(appId: "appID")?.isActive = false

// Or, by getting the sole registered Insights instance
Insights.shared?.isActive = false
```

### Logging and debuging

In case you want to check if the metric was sent correctly, you need to enable the logging first

```swift
Insights.shared(appId: "appId")?.isLoggingEnabled = true
```

After you enabled it, you can check the output for success or error messages


### Events flush delay

By default the client transmits tracked events every 30 seconds. You can customize this delay by changing `flushDelay` value (in seconds) as follows:

```swift
Insights.flushDelay = 60
```

#### Setting API region

By default each analytics API call is geo-routed so that each call targets the closest API. 
Today the analytics API suports two regions: United States and Germany. You can specify the region your prefer to use as follows: 

```swift
Insights.region = .de
```
